library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity zegar is port ( 
	jm: inout std_logic_vector(9 downto 0);
	dzm: inout std_logic_vector(5 downto 0);
	dzh: inout std_logic_vector(2 downto 1);
	jh: inout std_logic_vector(9 downto 0);
	ster_dk, szybko_wy, wolno_wy: out std_logic;
	clk_ref, res, mode, clk_set, res_a, set_a: in std_logic
	 );
end zegar;

architecture behavioral of zegar is
component licznik_jm port (
				clk : in std_logic;
	         clk_o : out std_logic;
            res : in std_logic;
            outp : inout std_logic_vector(9 downto 0)
);
end component licznik_jm;

component licznik_dzm port (
				clk : in std_logic;
	         clk_o : out std_logic;
            res : in std_logic;
            outp : inout std_logic_vector(5 downto 0)
);
end component licznik_dzm;

component licznik_h port (
			  clk : in std_logic;
           res, mode : in std_logic;
           outp_j : inout std_logic_vector(9 downto 0);
           outp_dz : inout std_logic_vector(2 downto 1)
);
end component licznik_h;

component mux_2_1 port (
	in0, in1, a : in std_logic;
	out_mux : out std_logic
);
end component mux_2_1;

component presk port (
	szybko, wolno, wy_1_min, wy_1Hz, wy_4Hz : out std_logic;
	res, clk: in std_logic 
);
end component presk;

signal a, d, clk_a, clk_min, clk_min_set: std_logic;
signal clk_0, clk_1, int_1Hz, int_4Hz, nie_clk_a: std_logic;

begin
licz_j_minut: licznik_jm port map (
		clk => clk_min_set,
      clk_o => clk_0,
      res => res,
      outp => jm
);

licz_d_minut: licznik_dzm port map (
		clk => clk_0,
      clk_o => clk_1,
      res => res,
      outp => dzm
);

licz_h: licznik_h port map (
		clk => clk_1,
      res => res,
	   mode => mode,
      outp_j => jh,
      outp_dz => dzh
);

mux_czasu: mux_2_1 port map (
	in0 => clk_min, 
	in1 => clk_set, 
	a => a, 
	out_mux => clk_min_set
);

mux_dwukropka: mux_2_1 port map (
	in0 => int_1Hz, 
	in1 => int_4Hz, 
	a => a, 
	out_mux => ster_dk
);

preskaler: presk port map (
	szybko => szybko_wy,
	wolno => wolno_wy,
	wy_1_min => clk_min,
	res => res,
	clk => clk_ref,
	wy_1Hz => int_1Hz,
	wy_4Hz => int_4Hz
);

-- przerzutnik D adresujacy multipleksery
-- wybor trybu pracy: uwstawianie/czas
d_ff:	process (clk_a, res)
		begin
			if res = '1' then 
			a <= '0';
			elsif clk_a = '1' and clk_a'event then
			a <= d;
			end if;
		d <= not a;
		end process d_ff;

-- przerzutnik RS likwidujacy drgania stykow
clk_a <= '0' when res_a = '1' else
			'1' when set_a = '1' else
			clk_a;

nie_clk_a <= '1' when res_a = '1' else
				'0' when set_a = '1' else
				clk_a;

end behavioral;
